/*********************************************************************
 *                
 * Filename:      obex_common.c
 * Version:       0.3
 * Description:   Utility-functions to act as a PUT server and client
 * Status:        Experimental.
 * Author:        Pontus Fuchs <pontus@tactel.se>
 * Created at:    Sat Nov 20 10:54:14 1999
 * Modified at:   Sun Nov 28 20:28:58 1999
 * Modified by:   Pontus Fuchs <pontus@tactel.se>
 * 
 *     Copyright (c) 1999 Dag Brattli, All Rights Reserved.
 *     
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License 
 *     along with this program; if not, write to the Free Software 
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *     MA 02111-1307 USA
 *     
 ********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>

#include <netinet/in.h>

#include <obex/obex.h>
#include "obex_put_common.h"

extern obex_t *handle;
extern volatile int finished;

volatile int busy = FALSE;
volatile int last_rsp = OBEX_RSP_BAD_REQUEST;

/*
 * Function build_object_from_file
 *
 *    Make an obex_object_t out of a file.
 *
 */
obex_object_t *build_object_from_file(char *filename)
{
	guint8 unicode_buf[200];
	gint namebuf_len;
 	obex_object_t *object;
	guint32 creator_id;
	int fd;
	int file_size;
	int actual;
	struct stat stats;
	char *name = NULL;
	guint8 *buf;

	printf("PUT file %s\n", filename);

	fd = open(filename, O_RDONLY, 0);
	if (fd == -1) {
		return NULL;
	}
	
	/*  Need to know the file length */
	fstat(fd, &stats);
	file_size = (int) stats.st_size;

	if(! (buf = g_malloc(file_size)) )	{
		return NULL;
	}

	actual = read(fd, buf, file_size);
	close(fd); 

	/* Set Memopad as the default creator ID */
	creator_id = MEMO_PAD;	

	/* Find the . in the filename */
	name = rindex(filename, '.');
	if (name) {
		name++;
		if (strcmp(name, "vcf") == 0) {
			printf( "This is a Address Book file\n");
			creator_id = ADDRESS_BOOK;
		} else if (strcmp(name, "vcs") == 0) {
			printf( "This is a Date Book file\n");
			creator_id = DATE_BOOK;
		} else if (strcmp(name, "txt") == 0) {
			printf("This is a Memo pad file\n");
			creator_id = MEMO_PAD;
		} else if (strcmp(name, "prc") == 0) {
			printf("This is a Pilot resource file\n");
			creator_id = PILOT_RESOURCE;
		}
	}
	/* Build object */
	object = OBEX_ObjectNew(handle, OBEX_CMD_PUT);

	namebuf_len = OBEX_CharToUnicode(unicode_buf, filename, sizeof(unicode_buf));

	OBEX_ObjectAddHeader(handle, object, OBEX_HEADER_NAME,
				(obex_headerdata_t) (guint8*) unicode_buf,
				namebuf_len, 0);

	OBEX_ObjectAddHeader(handle, object, OBEX_HEADER_LENGTH,
				(obex_headerdata_t) (guint32) actual,
				sizeof(guint32), 0);

	/* Optional header for win95 irxfer, allows date to be set on file */
	OBEX_ObjectAddHeader(handle, object, OBEX_HEADER_TIME2,
				(obex_headerdata_t) (guint32) stats.st_mtime,
				sizeof(guint32), 0);

	/* Optional header for Palm Pilot */
	OBEX_ObjectAddHeader(handle, object, HEADER_CREATOR_ID,
				(obex_headerdata_t) creator_id,
				sizeof(guint32), 0);

	OBEX_ObjectAddHeader(handle, object, OBEX_HEADER_BODY,
				(obex_headerdata_t) buf,
				actual, 0);

	g_free(buf);
	return object;
}


/*
 * Function safe_save_file ()
 *
 *    First remove path and add "/tmp/". Then save.
 *
 */
gint safe_save_file(gchar *name, guint8 *buf, gint len)
{
	gchar *s;
	gchar filename[255];
	gint fd;
	gint actual;

	sprintf( filename, "/tmp/");
	s = rindex(name, '/');
	if (s == NULL)
		s = name;
	else
		s++;

	strncat(filename, s, 250);
	fd = open(filename, O_RDWR | O_CREAT, DEFFILEMODE);
	if ( fd < 0) {
		perror( filename);
		return -1;
	}
	
	actual = write(fd, buf, len);
	close(fd);

	g_print( "irobex_palm3: wrote %s (%d bytes)\n", filename, actual);

	return actual;
}

/*
 * Function put_done()
 *
 *    Parse what we got from a PUT
 *
 */
void put_done(obex_object_t *object)
{
	obex_headerdata_t hv;
	guint8 hi;
	gint hlen;

	guint8 *body = NULL;
	gint body_len;
	gchar *name = NULL;
	gchar *namebuf = NULL;

	while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen))	{
		switch(hi)	{
		case OBEX_HEADER_BODY:
			body = hv.bs;
			body_len = hlen;
			break;
		case OBEX_HEADER_NAME:
			if( (namebuf = g_malloc(hlen / 2)))	{
				OBEX_UnicodeToChar(namebuf, hv.bs, hlen);
				name = namebuf;
			}
			break;

		case OBEX_HEADER_LENGTH:
			g_print("HEADER_LENGTH = %d\n", hv.bq4);
			break;

		case HEADER_CREATOR_ID:
			g_print("CREATORID = %#x\n", hv.bq4);
			break;
		
		default:
			g_print(__FUNCTION__ "() Skipped header %02x\n", hi);
		}
	}
	if(!body)	{
		g_print("Got a PUT without a body\n");
		return;
	}
	if(!name)	{
		g_print("Got a PUT without a name. Setting name to %s\n", name);
		name = "OBEX_PUT_Unknown_object";

	}
	safe_save_file(name, body, body_len);
	g_free(namebuf);
}


/*
 * Function server_indication()
 *
 * Called when a request is about to come or has come.
 *
 */
void server_request(obex_object_t *object, gint event, gint cmd)
{
	switch(cmd)	{
	case OBEX_CMD_PUT:
		if(event == OBEX_EV_REQ)	{
			OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
			put_done(object);
		}
		else /* OBEX_REQHINT */
			OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
		break;

	case OBEX_CMD_CONNECT:
		OBEX_ObjectSetRsp(object, OBEX_RSP_SUCCESS, OBEX_RSP_SUCCESS);
		break;
	case OBEX_CMD_DISCONNECT:
		OBEX_ObjectSetRsp(object, OBEX_RSP_SUCCESS, OBEX_RSP_SUCCESS);
		break;
	default:
		g_print(__FUNCTION__ "() Denied %02x request\n", cmd);
		OBEX_ObjectSetRsp(object, OBEX_RSP_NOT_IMPLEMENTED, OBEX_RSP_NOT_IMPLEMENTED);
		break;
	}
	return;
}

/*
 * Function client_done_indication()
 *
 * Called when a server-operation has finished
 *
 */
void server_done(obex_object_t *object, gint obex_cmd)
{
	/* Quit if DISCONNECT has finished */
	if(obex_cmd == OBEX_CMD_DISCONNECT)
		finished = 1;
}


/*
 * Function client_done()
 *
 * Called when a client-operation has finished
 *
 */
void client_done(obex_object_t *object, gint obex_cmd, gint obex_rsp)
{
	last_rsp = obex_rsp;
	busy = FALSE;
}

/*
 * Function obex_event ()
 *
 *    Called by the obex-layer when some event occurs.
 *
 */
void obex_event(obex_object_t *object, gint mode, gint event, gint obex_cmd, gint obex_rsp)
{

	switch (event)	{
	case OBEX_EV_PROGRESS:
		g_print("Made some progress...\n");
		break;
	case OBEX_EV_REQDONE:
		if(mode == OBEX_CLIENT)
			client_done(object, obex_cmd, obex_rsp);
		else
			server_done(object, obex_cmd);
		break;
	case OBEX_EV_REQ:
	case OBEX_EV_REQHINT:
		server_request(object, event, obex_cmd);
		break;
	case OBEX_EV_LINKERR:
		g_print("Link broken!\n");
		OBEX_TransportDisconnect(handle);
		break;
	default:
		g_print("Unknown event!\n");
		break;
	}
}

/*
 * Function wait_for_rsp()
 *
 *    Busy-wait for a request to finish!
 *
 */
int wait_for_rsp(gint async)
{
	int ret;

	if(!async)	{
		ret = OBEX_HandleInput(handle, 10);
		if (ret < 1)
			exit(2);
	}
	else	{
		while(busy)
			sleep(1);
	}
	return last_rsp;
}

/*
 * Function do_sync_resuest()
 *
 *    Execute an OBEX-request synchronously.
 *    This one really need to be more bullet-proof!
 */
int do_sync_request(obex_t *handle, obex_object_t *object, gint async)
{
	busy = TRUE;
	OBEX_Request(handle, object);

	return wait_for_rsp(async);
}
