/*********************************************************************
 *                
 * Filename:      obex_server.c
 * Version:       0.5
 * Description:   Handles server-like operations
 * Status:        Experimental.
 * Author:        Pontus Fuchs <pontus@tactel.se>
 * Created at:    Thu Nov 11 20:55:00 1999
 * Modified at:   Tue Nov 16 19:25:19 1999
 * Modified by:   Pontus Fuchs <pontus@tactel.se>
 * 
 *     Copyright (c) 1998-1999 Dag Brattli, All Rights Reserved.
 *      
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2 of the License, or (at your option) any later version.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *     MA  02111-1307  USA
 *     
 ********************************************************************/

#include <config.h>

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <obex_main.h>
#include <obex_header.h>
#include <obex_connect.h>


/*
 * Function obex_server ()
 *
 *    Handle server-operations
 *
 */
gint obex_server(obex_t *self, GNetBuf *msg, gint final)
{
	obex_common_hdr_t *hdr;
	gint opcode;
	gint ret;

	DEBUG(4, __FUNCTION__ "()\n");

	hdr = (obex_common_hdr_t *) msg->data;
	opcode = hdr->opcode & ~OBEX_FINAL;

	if(opcode == OBEX_CMD_CONNECT)	{
		/* Connect needs some special treatment */
		DEBUG(4, __FUNCTION__ "() Got CMD_CONNECT\n");
		if(obex_parse_connect_header(self, msg) < 0)	{
			obex_response_request(self, OBEX_RSP_BAD_REQUEST);
			obex_deliver_event(self, OBEX_CLIENT, OBEX_EV_PARSEERR, self->object->cmd, 0, TRUE);
			return -1;
		}

	}

	if(! self->object)	{
		DEBUG(4, __FUNCTION__ "() First package of a request\n");
		/* This is the first package of a request */
		self->object = obex_object_new();

		if(! self->object) {
			DEBUG(1, __FUNCTION__ "() Allocation of object failed!\n");
			obex_response_request(self, OBEX_RSP_INTERNAL_SERVER_ERROR);
			return -1;
		}
		
		if(obex_object_receive(self->object, msg) < 0)	{
			obex_response_request(self, OBEX_RSP_BAD_REQUEST);
			obex_deliver_event(self, OBEX_CLIENT, OBEX_EV_PARSEERR, self->object->cmd, 0, TRUE);
			return -1;
		}

		if(! final)	{
			/* Hint app that something is about to come so that
			   the app can deny a PUT-like request */
			obex_deliver_event(self, OBEX_SERVER, OBEX_EV_REQHINT, opcode, 0, FALSE);
		}
	}
	else	{
		obex_object_receive(self->object, msg);
	}

	if( ( self->object->app_is_called == FALSE ) && final)	{
		DEBUG(4, __FUNCTION__ "() We got a request!\n");
		self->object->app_is_called = TRUE;
		/* More connect-magic woodoo stuff */
		if(opcode == OBEX_CMD_CONNECT)
			obex_insert_connectframe(self, self->object);

		obex_deliver_event(self, OBEX_SERVER, OBEX_EV_REQ, opcode, 0, FALSE);
	}

	ret = obex_object_send(self, self->object, self->object->app_is_called);
	if(ret == 0)
		obex_deliver_event(self, OBEX_SERVER, OBEX_EV_PROGRESS, opcode, 0, FALSE);
	else if(ret > 0) {
		DEBUG(3, __FUNCTION__ "() Done doing what we did!\n");

		if(opcode == OBEX_CMD_DISCONNECT)	{
			DEBUG(2, __FUNCTION__ "() CMD_DISCONNECT done. Resetting MTU!\n");
			self->mtu_tx = OBEX_MINIMUM_MTU;
		}
		obex_deliver_event(self, OBEX_SERVER, OBEX_EV_REQDONE, opcode, 0, TRUE);
	}
	else
		obex_deliver_event(self, OBEX_SERVER, OBEX_EV_LINKERR, opcode, 0, TRUE);
	return 0;
}
